package com.lqkj.camera.util;

import com.lqkj.camera.config.CameraConfig;
import com.lqkj.camera.model.CameraVo;
import org.bytedeco.ffmpeg.avcodec.AVPacket;
import org.bytedeco.ffmpeg.avformat.AVFormatContext;
import org.bytedeco.javacv.FFmpegFrameGrabber;
import org.bytedeco.javacv.FFmpegFrameRecorder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static org.bytedeco.ffmpeg.global.avcodec.av_packet_unref;

public class CameraPush {
    private final static Logger logger = LoggerFactory.getLogger(CameraPush.class);
    private static CameraConfig cameraConfig;

    // 通过applicationContext上下文获取Config类
    public static void setApplicationContext(ApplicationContext applicationContext) {
        cameraConfig = applicationContext.getBean(CameraConfig.class);
    }

    protected FFmpegFrameGrabber grabber = null;// 解码器
    protected FFmpegFrameRecorder record = null;// 编码器
    int width;// 视频像素宽
    int height;// 视频像素高

    // 视频参数
    protected int audiocodecid;
    protected int codecid;
    protected double framerate;// 帧率
    protected int bitrate;// 比特率

    // 音频参数
    // 想要录制音频，这三个参数必须有：audioChannels > 0 && audioBitrate > 0 && sampleRate > 0
    private int audioChannels;
    private int audioBitrate;
    private int sampleRate;

    private CameraVo cameraVo;

    public CameraPush() {
        super();
    }

    public CameraPush(CameraVo cameraVo) {
        this.cameraVo = cameraVo;
    }


    /**
     * 选择视频源
     */
    public CameraPush from() throws Exception {
        // 采集/抓取器
        grabber = FFmpegFrameGrabber.createDefault(cameraVo.getRtsp());

        //校验相机和rtmp服务器是否能访问
        Socket rtspSocket = new Socket();
        Socket rtmpSocket = new Socket();
        // 建立TCP Scoket连接，超时时间2s，如果成功继续执行，否则return
        logger.debug("******   TCPCheck    BEGIN   ******");
        String cameraIp = "";
        String cameraPort = "";
        if(cameraConfig.getTcpConnectCheck()){
            try {
                Pattern pattern = Pattern.compile("(\\d+\\.\\d+\\.\\d+\\.\\d+)\\:(\\d+)");
                Matcher matcher = pattern.matcher(cameraVo.getRtsp());
                while(matcher.find()) {
                    cameraIp = matcher.group(1);
                    cameraPort = matcher.group(2);
                }
                rtspSocket.connect(new InetSocketAddress(cameraIp, Integer.parseInt(cameraPort.trim())), 2000);
            } catch (IOException e) {
                grabber.stop();
                grabber.close();
                rtspSocket.close();
                logger.error("与拉流IP：   " + cameraIp + "   端口：  " + cameraPort +"    建立TCP连接失败！");
                return null;
            }
        }
        try {
            rtmpSocket.connect(new InetSocketAddress(cameraConfig.getPushHost(),
                    cameraConfig.getPushPort()), 2000);
        } catch (IOException e) {
            grabber.stop();
            grabber.close();
            rtspSocket.close();
            logger.error("与推流IP：   " + cameraConfig.getPushHost() + "   端口：   " +cameraConfig.getPushPort() + " 建立TCP连接失败！");
            return null;
        }
        logger.debug("******   TCPCheck    END     ******");

        if (cameraVo.getRtsp().indexOf("rtsp") >= 0) {
            grabber.setOption("rtsp_transport", "tcp");// tcp用于解决丢包问题
            grabber.setOption("stimeout", String.valueOf(15 * 1000000));
        }else{
            grabber.setOption("rw_timeout", String.valueOf(15 * 1000000));
        }
        try {
            logger.debug("******   grabber.start()    BEGIN   ******");
            grabber.startUnsafe();
            logger.debug("******   grabber.start()    END     ******");

            // 开始之后ffmpeg会采集视频信息，之后就可以获取音视频信息

            width = grabber.getImageWidth();
            height = grabber.getImageHeight();
            // 若视频像素值为0，说明拉流异常，程序结束
            if (width == 0 && height == 0) {
                logger.error(cameraVo.getRtsp() + "  拉流异常！");
                grabber.stop();
                grabber.close();
                return null;
            }
            // 视频参数
            audiocodecid = grabber.getAudioCodec();
            codecid = grabber.getVideoCodec();
            framerate = grabber.getVideoFrameRate();// 帧率
            bitrate = grabber.getVideoBitrate();// 比特率
            // 音频参数
            // 想要录制音频，这三个参数必须有：audioChannels > 0 && audioBitrate > 0 && sampleRate > 0
            audioChannels = grabber.getAudioChannels();
            audioBitrate = grabber.getAudioBitrate();
            if (audioBitrate < 1) {
                audioBitrate = 128 * 1000;// 默认音频比特率
            }
            sampleRate = grabber.getSampleRate();
        } catch (org.bytedeco.javacv.FrameGrabber.Exception e) {
            logger.error("ffmpeg错误信息：", e);
            grabber.stop();
            grabber.close();
            return null;
        }

        return this;
    }

    /**
     * 选择输出
     */
    public CameraPush to() throws Exception {
        // 录制/推流器
        record = new FFmpegFrameRecorder(cameraVo.getRtmp(), width, height);
        record.setVideoOption("crf", "28");// 画面质量参数，0~51；18~28是一个合理范围
        record.setGopSize(2);
        record.setFrameRate(framerate);
        record.setVideoBitrate(bitrate);

        record.setAudioChannels(audioChannels);
        record.setAudioBitrate(audioBitrate);
        record.setSampleRate(sampleRate);
        AVFormatContext fc = null;
        if (cameraVo.getRtmp().indexOf("rtmp") >= 0 || cameraVo.getRtmp().indexOf("flv") > 0) {
            // 封装格式flv
            record.setFormat("flv");
            record.setAudioCodecName("aac");
            record.setVideoCodec(codecid);
            fc = grabber.getFormatContext();
        }
        try {
            record.start(fc);
        } catch (Exception e) {
            logger.error(cameraVo.getRtsp() + "  推流异常！");
            logger.error("ffmpeg错误信息：", e);
            grabber.stop();
            grabber.close();
            record.stop();
            record.close();
            return null;
        }
        return this;

    }

    /**
     * 转封装
     */
    public CameraPush go(Thread nowThread)
            throws org.bytedeco.javacv.FrameGrabber.Exception, org.bytedeco.javacv.FrameRecorder.Exception {
        long err_index = 0;// 采集或推流导致的错误次数
        // 连续五次没有采集到帧则认为视频采集结束，程序错误次数超过5次即中断程序
        logger.info(cameraVo.getRtsp() + " 开始推流...");
        // 释放探测时缓存下来的数据帧，避免pts初始值不为0导致画面延时
        grabber.flush();
        for (int no_frame_index = 0; no_frame_index < 5 || err_index < 5; ) {
            try {
                // 用于中断线程时，结束该循环
                nowThread.sleep(1);
                AVPacket pkt = null;
                // 获取没有解码的音视频帧
                pkt = grabber.grabPacket();
                if (pkt == null || pkt.size() <= 0 || pkt.data() == null) {
                    // 空包记录次数跳过
                    no_frame_index++;
                    err_index++;
                    continue;
                }
                // 不需要编码直接把音视频帧推出去
                err_index += (record.recordPacket(pkt) ? 0 : 1);

                av_packet_unref(pkt);
            } catch (InterruptedException e) {
                e.printStackTrace();
                // 销毁构造器
                grabber.stop();
                grabber.close();
                record.stop();
                record.close();
                logger.info(cameraVo.getRtsp() + " 中断推流成功！");
                break;
            } catch (org.bytedeco.javacv.FrameGrabber.Exception e) {
                err_index++;
            } catch (org.bytedeco.javacv.FrameRecorder.Exception e) {
                err_index++;
            }
        }
        // 程序正常结束销毁构造器
        grabber.stop();
        grabber.close();
        record.stop();
        record.close();
        logger.info(cameraVo.getRtsp() + " 推流结束...");
        return this;
    }
}
