package com.lqkj.system.controller;

import com.lqkj.ApiVersion;
import com.lqkj.common.annotation.Log;
import com.lqkj.common.constant.UserConstants;
import com.lqkj.common.core.controller.BaseController;
import com.lqkj.common.core.model.ResultUtil;
import com.lqkj.common.enums.BusinessType;
import com.lqkj.common.utils.StringUtils;
import com.lqkj.system.entity.SysDept;
import com.lqkj.system.service.SysDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Iterator;
import java.util.List;

/**
 * 部门信息
 * 
 * @author lh
 */
@Api(tags = {"部门信息"})
@RestController
@RequestMapping("/" + ApiVersion.V1 + "/system/dept")
public class SysDeptController extends BaseController
{
    @Autowired
    private SysDeptService deptService;

    /**
     * 获取部门列表
     */
    @ApiOperation("获取部门列表")
    @PreAuthorize("@ss.hasPermi('system:dept:list')")
    @GetMapping("/list")
    public ResultUtil list(SysDept dept)
    {
        List<SysDept> depts = deptService.selectDeptList(dept);
        return ResultUtil.success(depts);
    }

    /**
     * 查询部门列表（排除节点）
     */
    @ApiOperation("查询部门列表（排除节点）")
    @PreAuthorize("@ss.hasPermi('system:dept:list')")
    @GetMapping("/list/exclude/{deptId}")
    public ResultUtil excludeChild(@PathVariable(value = "deptId", required = false) Long deptId)
    {
        List<SysDept> depts = deptService.selectDeptList(new SysDept());
        Iterator<SysDept> it = depts.iterator();
        while (it.hasNext())
        {
            SysDept d = (SysDept) it.next();
            if (d.getDeptId().intValue() == deptId
                    || ArrayUtils.contains(StringUtils.split(d.getAncestors(), ","), deptId + ""))
            {
                it.remove();
            }
        }
        return ResultUtil.success(depts);
    }

    /**
     * 根据部门编号获取详细信息
     */
    @ApiOperation("根据部门编号获取详细信息")
    @PreAuthorize("@ss.hasPermi('system:dept:query')")
    @GetMapping(value = "/{deptId}")
    public ResultUtil getInfo(@PathVariable Integer deptId)
    {
        deptService.checkDeptDataScope(deptId);
        return ResultUtil.success(deptService.selectDeptById(deptId));
    }

    /**
     * 获取部门下拉树列表
     */
    @ApiOperation("获取部门下拉树列表")
    @GetMapping("/treeselect")
    public ResultUtil treeselect(SysDept dept)
    {
        List<SysDept> depts = deptService.selectDeptList(dept);
        return ResultUtil.success(deptService.buildDeptTreeSelect(depts));
    }

    /**
     * 加载对应角色部门列表树
     */
    @ApiOperation("加载对应角色部门列表树")
    @GetMapping(value = "/roleDeptTreeselect/{roleId}")
    public ResultUtil roleDeptTreeselect(@PathVariable("roleId") Integer roleId)
    {
        List<SysDept> depts = deptService.selectDeptList(new SysDept());
        ResultUtil result = ResultUtil.success();
        result.put("checkedKeys", deptService.selectDeptListByRoleId(roleId));
        result.put("depts", deptService.buildDeptTreeSelect(depts));
        return result;
    }

    /**
     * 新增部门
     */
    @ApiOperation("新增部门")
    @PreAuthorize("@ss.hasPermi('system:dept:add')")
    @Log(title = "部门管理", businessType = BusinessType.INSERT)
    @PostMapping
    public ResultUtil add(@Validated @RequestBody SysDept dept)
    {
        if (UserConstants.NOT_UNIQUE.equals(deptService.checkDeptNameUnique(dept)))
        {
            return ResultUtil.error("新增部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        }
        dept.setCreateBy(getNickName());
        return resultByRows(deptService.insertDept(dept));
    }

    /**
     * 修改部门
     */
    @ApiOperation("修改部门")
    @PreAuthorize("@ss.hasPermi('system:dept:edit')")
    @Log(title = "部门管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public ResultUtil edit(@Validated @RequestBody SysDept dept)
    {
        if (UserConstants.NOT_UNIQUE.equals(deptService.checkDeptNameUnique(dept)))
        {
            return ResultUtil.error("修改部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        }
        else if (dept.getParentId().equals(dept.getDeptId()))
        {
            return ResultUtil.error("修改部门'" + dept.getDeptName() + "'失败，上级部门不能是自己");
        }
        else if (!dept.getStatus()
                && deptService.selectNormalChildrenDeptById(dept.getDeptId()) > 0)
        {
            return ResultUtil.error("该部门包含未停用的子部门！");
        }
        dept.setUpdateBy(getNickName());
        return resultByRows(deptService.updateDept(dept));
    }

    /**
     * 删除部门
     */
    @ApiOperation("删除部门")
    @PreAuthorize("@ss.hasPermi('system:dept:remove')")
    @Log(title = "部门管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{deptId}")
    public ResultUtil remove(@PathVariable Integer deptId)
    {
        if (deptService.hasChildByDeptId(deptId))
        {
            return ResultUtil.error("存在下级部门,不允许删除");
        }
        if (deptService.checkDeptExistUser(deptId))
        {
            return ResultUtil.error("部门存在用户,不允许删除");
        }
        return resultByRows(deptService.deleteDeptById(deptId));
    }

    /**
     * 获取部门列表(不做权限过滤)
     */
    @ApiOperation("获取所有部门列表(不做数据权限过滤)")
    @GetMapping("/listDeptSelect")
    public ResultUtil listDeptSelect(SysDept dept){
        return ResultUtil.success(deptService.buildDeptTreeSelect(deptService.selectDeptAll(dept)));
    }
    /**
     * 获取部门列表(做权限过滤)
     */
    @ApiOperation("获取所有部门列表(做数据权限过滤)")
    @GetMapping("/listDeptSelectByPermi")
    public ResultUtil listDeptSelectByPermi(SysDept dept){
        return ResultUtil.success(deptService.selectDeptUserByPermi(dept));
    }
    /**
     * 获取部门和用户列表(不做权限过滤)
     */
    @ApiOperation("获取部门和用户列表(不做数据权限过滤)")
    @GetMapping("/listDeptUserSelect")
    public ResultUtil listDeptUserSelect(SysDept dept){
        return ResultUtil.success(deptService.selectDeptUserAll(dept));
    }

}
